"""
Module contains functions for mongodb
"""
import importlib
import tornado.log

from torskel.libs.str_consts import INIT_MONGO_LABEL


# pylint: disable=C0103
logger = tornado.log.gen_log


# pylint: disable=W1203
def get_mongo_pool(**kwargs):
    """
    Create connections pool to mongo
    :param kwargs:
    :return:
    """
    mongo_db_name = kwargs.get('mongo_db_name', None)
    mongo_user = kwargs.get('mongo_user', None)
    mongo_psw = kwargs.get('mongo_psw', None)
    mongo_auth_db_name = kwargs.get('mongo_auth_db_name', None)
    mongo_server = kwargs.get('mongo_server', 'localhost')
    mongo_port = kwargs.get('mongo_port', 27017)
    mongo_min_pool_size = kwargs.get('mongo_min_pool_size', 5)
    mongo_max_pool_size = kwargs.get('mongo_max_pool_size', 10)
    con_str = kwargs.get('con_str', None)
    db_name = kwargs.get('db_name', None)

    try:
        motor = importlib.import_module('motor')
        logger.info('Init MongoDB pool...')
        if con_str is None:
            logger.info(f'[{INIT_MONGO_LABEL}] MONGO_SRV={mongo_server} '
                        f'MONGO_PORT={mongo_port}')

            logger.info(f'[{INIT_MONGO_LABEL}]MONGO_USER={mongo_user}')

            logger.info(f'[{INIT_MONGO_LABEL}]'
                        f'MONGO_DB_NAME={mongo_db_name}')

            logger.info(f'[{INIT_MONGO_LABEL}]MONGO_AUTH_DB_NAME'
                        f'={mongo_auth_db_name}')

            logger.info(f'[{INIT_MONGO_LABEL}]MONGO_MIN_POOL_SIZE'
                        f'={mongo_min_pool_size}')

            logger.info(f'[{INIT_MONGO_LABEL}]MONGO_MAX_POOL_SIZE'
                        f'={mongo_max_pool_size}')
            mongo_uri = f'mongodb://{mongo_user}:{mongo_psw}' \
                        f'@{mongo_server}:{mongo_port}' \
                        f'/{mongo_db_name}' \
                        f'?authSource={mongo_auth_db_name}' \
                        f'&minPoolSize={mongo_min_pool_size}' \
                        f'&maxPoolSize={mongo_max_pool_size}'

            res = motor.motor_tornado.MotorClient(mongo_uri)[mongo_db_name]
        else:
            if isinstance(con_str, str) and isinstance(db_name, str):
                res = motor.motor_tornado.MotorClient(con_str)[db_name]
            else:
                res = None
    except ImportError:
        raise ModuleNotFoundError('Required package motor is missing')
    return res


async def bulk_mongo_insert(db, collection_name, bulk_list):
    """
    Bulk insert into collection
    :param db: database name
    :param collection_name:  collection name
    :param bulk_list: list of documents
    :return:
    """
    await db[collection_name].insert_many(bulk_list)



"""
Module contains constants
"""

INIT_REDIS_LABEL = 'INIT_REDIS'
INIT_MONGO_LABEL = 'INIT_MONGO'

EVENTS_USER_AGENT = 'user_agent'
EVENTS_DATE = 'date_event'
EVENTS_IP = 'user_ip'
EVENTS_URL = 'handler_url'
EVENTS_SRV_NAME = 'server_name'
EVENTS_METHOD = 'method'



"""
    Request handler class with auth
"""
from torskel.torskel_handler import TorskelHandler
from torskel.libs.auth.jwt import jwtauth


# pylint: disable=W0223
@jwtauth
class TorskelSecuredHandler(TorskelHandler):
    """
    RequestHandler class with auth
    """



"""
Logging routines
"""

import tornado.log

LOG_MSG_DEBUG_TMPL = '%s %s'


class TorskelLogMixin:
    """
    Mixin with logging function
    """

    def __init__(self):
        self.logger = tornado.log.gen_log
        self.log_msg_tmpl = LOG_MSG_DEBUG_TMPL

        # ################### #
        #  Logging functions  #
        # ################### #

    def _get_log_msg(self, msg, grep_label=''):
        """
        Make message by template
        :param msg: message
        :param grep_label: label for grep
        :return: compiled message
        """
        try:
            res = self.log_msg_tmpl % (grep_label, msg)
        except TypeError:
            res = msg
        return res

    def log_debug(self, msg, grep_label=''):
        """
        Log debug message
        :param msg: message
        :param grep_label: label for grep
        :return:
        """
        self.logger.debug(self._get_log_msg(msg, grep_label))

    def log_info(self, msg, grep_label=''):
        """
        Log info message
        :param msg: message
        :param grep_label: label for grep
        :return:
        """
        self.logger.info(self._get_log_msg(msg, grep_label))

    def log_err(self, msg, grep_label=''):
        """
        Log error
        :param msg: message
        :param grep_label: label for grep
        :return:
        """
        self.logger.error(self._get_log_msg(msg, grep_label))

    def log_exc(self, msg, grep_label=''):
        """
        Log exception
        :param msg: message
        :param grep_label: label for grep
        :return:
        """
        self.logger.exception(self._get_log_msg(msg, grep_label))



"""
Module contains startup methods
"""
# pylint: disable=C0103
import logging
import importlib
from tornado.options import options
from tornado.httpserver import HTTPServer
import tornado.log


# pylint: disable=C0103
logger = tornado.log.gen_log
# hotfix for windows
try:
    from tornado.netutil import bind_unix_socket
    no_unix_socket = False
except ImportError:
    no_unix_socket = True


def _configure_graylog():
    """
    Configuring graylog's handlers
    :return:
    """
    logger.info('Configuring graylog host=%s port=%s', options.graylog_host,
                options.graylog_port)
    try:
        graypy = importlib.import_module('graypy')
        handler = graypy.GELFHandler(
            options.graylog_host,
            options.graylog_port,
            localname=options.srv_name
        )
        logging.getLogger("tornado.access").addHandler(handler)
        logging.getLogger("tornado.application").addHandler(handler)
        logging.getLogger("tornado.general").addHandler(handler)
    except ImportError:
        raise ModuleNotFoundError('Required package graypy is missing')


def server_init(server):
    """
    Initializing an application on a port or socket depending on the settings
    :param server:
    :return: None
    """
    if options.use_graylog:
        _configure_graylog()

    if options.run_on_socket and not no_unix_socket:

        unix_socket = bind_unix_socket(options.socket_path, 0o666)
        http_server = HTTPServer(server)
        http_server.add_socket(unix_socket)
        server.log_info(f'Running on socket {options.socket_path}')
    else:

        server.log_info(f'Running on port {options.port}')
        server.listen(options.port)



from torskel.torskel_handler import TorskelHandler


class TestPage(TorskelHandler):
    def get(self):
        self.write('Hi')
        self.finish()


class TestUserIp(TorskelHandler):
    def get(self):
        self.write(self.get_user_ip())
        self.finish()



"""
Module with RequestHandler
"""

# pylint: disable=W0511
import json
from datetime import datetime

import xmltodict

from user_agents import parse

from tornado.options import options
from tornado.web import RequestHandler
from torskel.str_utils import get_hash_str
from torskel.str_utils import is_hash_str
from torskel.torskel_mixins.log_mix import TorskelLogMixin
from torskel.libs.str_consts import EVENTS_USER_AGENT
from torskel.libs.str_consts import EVENTS_DATE
from torskel.libs.str_consts import EVENTS_IP, EVENTS_METHOD
from torskel.libs.str_consts import EVENTS_URL, EVENTS_SRV_NAME
from torskel.str_utils import default_json_dt
from torskel.libs.auth.jwt import jwt_encode


# pylint: disable=W0223
class TorskelHandler(RequestHandler, TorskelLogMixin):
    """
    Request Handler class
    """
    def __init__(self, application, request, **kwargs):
        super(TorskelHandler, self).__init__(application, request, **kwargs)
        # filter dict by list of keys
        # pylint: disable=R1717
        self.filter_dict = lambda x, y: dict(
            [(i, x[i]) for i in x if i in set(y)]
        )

    @staticmethod
    def get_hash_str(value, alg='sha224'):
        """
        Return hash by string
        :param value: source string
        :param alg: algorithm, default sha224
        :return: hash
        """
        return get_hash_str(value, alg)

    @staticmethod
    def is_hash_str(value):
        """
        Checks whether the hash string is a string of any algorithm
        :param value: hash string
        :return: boolean
        """
        return is_hash_str(value)

    def encode_jwt_token(self, payload: dict = None):
        """
        Encode JWT token from payload
        :param payload:
        :return:
        """
        if payload is None:
            payload = {}
        return jwt_encode(self.application.get_secret_key(), payload)

    def get_req_args(self, args_list=None):
        """
        Get list of params from request
        :param args_list: list of params
        :return: list of values
        """
        if not args_list:
            args_list = []
        return [self.get_argument(x, False) for x in args_list]

    def get_req_args_dict(self):
        """
        Return dict of request arguments
        :return: dict
        """

        try:
            res = {
                k: ''.join([i.decode('utf-8') for i in v])
                for k, v in self.request.arguments.items()
            }
        except UnicodeEncodeError:
            self.log_exc('get_req_args_dict failed')
            res = {}
        return res

    def get_user_ip(self):
        """
        Return user ip from request object
        :return: user_ip
        """
        x_real_ip = self.request.headers.get("X-Real-IP")
        remote_ip = x_real_ip or self.request.remote_ip
        return remote_ip

    def _handle_request_exception(self, e):
        super(TorskelHandler, self)._handle_request_exception(e)
        msg = '''handler classname = %s \n
                request = %s \n
                exception = %s \n'''

        self.log_exc(msg % (type(self).__name__, self.request, e))

    async def http_request_get(self, url, **kwargs):
        """
        async http request. Method GET
        :param url: url
        :param from_json: boolean, convert response to dict
        :return: response
        """

        return await self.application.http_request_get(url, **kwargs)

    async def http_request_post(self, url, body, **kwargs):
        """
        async http request. Method POST
        :param url: url
        :param body: dict with POST-params
        :param from_json: boolean, convert response to dict
        :return: response
        """
        return await self.application.http_request_post(url, body, **kwargs)

    # TODO refact add params to kwargs
    async def set_redis_exp_val(self, key, val, exp=None, **kwargs):
        """
        Write value to redis
        :param key: key
        :param val: value
        :param exp: Expire time in seconds
        :param convert_to_json: bool
        :param use_json_utils: bool use json utils from bson
        """
        await self.application.set_redis_exp_val(key, val, exp, **kwargs)

    async def del_redis_val(self, key):
        """
        delete value from redis by key
        :param key: key

        """
        await self.application.del_redis_val(key)

    async def get_redis_val(self, key, **kwargs):
        """
        get value from redis by key
        :param key: key
        :param from_json: loads from json
        :param use_json_utils: bool use json utils from bson
        :return: value
        """
        res = await self.application.get_redis_val(key, **kwargs)
        return res

    def get_current_url(self):
        """
        Get current handler url for urls named by class name
        :return: str
        """
        return self.reverse_url(self.__class__.__name__)

    def get_user_agent(self) -> str:
        """
        Defines the device, OS and user browser by the User-Agent header
        :return: str
        """
        try:
            ua_string = self.request.headers["User-Agent"]
            user_agent = str(parse(ua_string))
        except KeyError:
            user_agent = 'Unknown'
        self.log_debug(user_agent, grep_label='USER_AGENT')
        return user_agent

    def get_user_lang_list(self) -> list:
        """
        Returns a list of the language codes configured in the client's browser
        :return: language codes
        """
        try:
            languages = self.request.headers["Accept-Language"].split(",")
            res = [
                language.strip().split(";")[0][:2] for language in languages
            ]
        except TypeError:
            res = [options.default_local_language]
        except IndexError:
            res = [options.default_local_language]
        return res

    def add_log_event(self, event=None, use_legacy_event=True):
        """
        Add event into LOgWriter queue
        :param event:
        :param use_legacy_event:
        :return:
        """
        if event is None:
            event = {}
        if use_legacy_event:
            legacy_event = self.get_event_skeleton(options.use_lite_event)
        else:
            legacy_event = {}

        if isinstance(event, dict):
            legacy_event.update(event)
            compl_event = legacy_event
        else:
            compl_event = legacy_event

        if compl_event:
            self.application.event_writer.add_log_event(compl_event)

    def get_event_skeleton(self, lite_event=False):
        """
        Returns skeleton of event for logging
        :param lite_event: flag for using lite event dict
        :return: dict
        """
        user_agent = self.get_user_agent()
        url = self.request.uri
        try:
            user_agent = user_agent[:128]
            url = url[:128]
        except TypeError:
            pass

        # pylint: disable=C0103
        ip = self.get_user_ip()
        res = {
            EVENTS_DATE: datetime.now(),
            EVENTS_USER_AGENT: user_agent,
            EVENTS_IP: ip,
        }

        if not lite_event:
            add_info = {
                EVENTS_URL: url,
                EVENTS_SRV_NAME: self.application.server_name,
                EVENTS_METHOD: self.request.method,

            }
            res.update(add_info)

        return res

    @staticmethod
    def default_json_dt(json_object):
        """
        Returns hook for formatting date in JSON
        :param o: json_object
        :return:
        """
        return default_json_dt(json_object)

    @staticmethod
    def get_result_dict(code=0, msg='', additional_dict=None, **kwargs):
        """
        Return result dict like {'resultCode': 0, 'resultMessage': 'OK"}
        :param code: code of result
        :param msg: message of result
        :param additional_dict: additional data if needed
        :param kwargs:
            result_code_key overrides value of resultCode key
            res_msg_key overrides value of resultMessage key
            to_json  convert dict to json
        :return: dict
        """
        if additional_dict is None:
            additional_dict = {}
        res_code_key = kwargs.get('result_code_key', 'resultCode')
        res_msg_key = kwargs.get('res_msg_key', 'resultMessage')
        res = {res_code_key: code, res_msg_key: msg}
        res.update(additional_dict)
        if kwargs.get('to_json', False):
            res = json.dumps(res)
        return res

    @staticmethod
    def get_result_xml(code=0, msg='', additional_dict=None, **kwargs):
        """
        Return xml like
        <root>
            <resultCode>-1</resultCode>
            <resultMessage>Invalid params</resultMessage>
        </root>
        :param code: code of result
        :param msg: message of result
        :param additional_dict: additional data if needed
        :param kwargs: root_key overrides name of root node
        :return: xml
        """
        root_key = kwargs.get('root_key', 'Result')
        res_dict = TorskelHandler.get_result_dict(code, msg, additional_dict)
        output_dict = {root_key: res_dict}
        result = xmltodict.unparse(output_dict)
        return result



"""
Module with basic tornado application class
"""

# pylint: disable=W0511
import json
import os.path
import importlib
import logging.handlers
from urllib.parse import urlencode
import tornado.log
import tornado.web
import tornado.httpclient
from tornado.options import options
from tornado.web import Application
from tornado.httpclient import AsyncHTTPClient
import xmltodict

from torskel.torskel_mixins.redis_mixin import RedisApplicationMixin
from torskel.torskel_mixins.log_mix import TorskelLogMixin
from torskel.libs.db_utils.mongo import get_mongo_pool
from torskel.libs.db_utils.mongo import bulk_mongo_insert
from torskel.libs.str_consts import INIT_REDIS_LABEL
from torskel.libs.event_controller import TorskelEventLogController
from torskel.libs.startup import server_init

# server params
options.define('debug', default=True, help='debug mode', type=bool)
options.define("port", default=8888, help="run on the given port", type=int)
options.define("srv_name", 'LOCAL', help="Server verbose name", type=str)
options.define("run_on_socket", False, help="Run on socket", type=bool)
options.define("socket_path", None, help="Path to unix-socket", type=str)


# xml-rpc
options.define(
    'use_xmlrpc', default=False, help='use xmlrpc client', type=bool
)
options.define("max_xmlrpc_clients", default=10, type=int)

# http-client params
options.define("max_http_clients", default=100, type=int)
options.define("http_client_timeout", default=30, type=int)
options.define("use_curl_http_client", default=False, type=bool)

# mail logger params
options.define('use_mail_logging', default=False, help='SMTP log handler',
               type=bool)
options.define("log_mail_subj", default='', type=str)
options.define("log_mail_from", default='', type=str)
options.define("log_mail_to", default=[], type=list)
options.define("log_mail_host", default='', type=str)
options.define("log_mail_user", default='', type=str)
options.define("log_mail_psw", default='', type=str)

# mongodb params
options.define("use_mongo", default=False, help="use mongodb", type=bool)
options.define("mongo_server", default="localhost", type=str)
options.define("mongo_port", default=27017, type=int)
options.define("mongo_db_name", type=str)
options.define("mongo_auth_db_name", default=options.mongo_db_name, type=str)
options.define("mongo_user", type=str)
options.define("mongo_psw", type=str)
options.define("mongo_min_pool_size", default=5, type=int)
options.define("mongo_max_pool_size", default=10, type=int)

# redis params
options.define('use_redis', default=False, help='use redis', type=bool)
options.define('use_redis_socket', default=True,
               help='connection to redis unixsocket file', type=bool)
options.define("redis_min_con", default=5, type=int)
options.define("redis_max_con", default=10, type=int)
options.define("redis_host", default='127.0.0.1', type=str)
options.define("redis_port", default=6379, type=int)
options.define("redis_socket", default='/var/run/redis/redis.sock', type=str)
options.define("redis_psw", default='', type=str)
options.define("redis_db", default=-1, type=int)

# events writer params
options.define('use_events_writer', default=False, help='use_events_writer',
               type=bool)
options.define('show_log_event_writer', default=False,
               type=bool)
options.define('use_lite_event', default=False, type=bool)
options.define("task_list_size", default=10, type=int)
options.define("writer_period", default=1000*10, type=int)
options.define("events_collection_name", default='user_events', type=str)

# user language settings
options.define("default_local_language", default='en', type=str)
options.define("default_international_language", default='en', type=str)

# graylog options

options.define('use_graylog', default=False, help='use graylog', type=bool)
options.define("graylog_host", default='127.0.0.1', type=str)
options.define("graylog_port", default=12201, type=int)


class TorskelServer(Application, RedisApplicationMixin, TorskelLogMixin):
    """
    Base class of tornado application. Contains methods for work with redis,
    mongoDB and etc
    """
    def __init__(self, handlers, root_dir=None, static_path=None,
                 template_path=None, **settings):
        self.log_msg_tmpl = '%s %s'

        # TODO add valiate paths
        create_http_client = settings.get('create_http_client', True)
        if root_dir is not None:
            app_static_dir = os.path.join(root_dir, "static") \
                if static_path is None else static_path

            app_template_dir = os.path.join(root_dir, "templates") \
                if template_path is None else template_path
        else:
            app_template_dir = app_static_dir = None

        super().__init__(handlers, static_path=app_static_dir,
                         template_path=app_template_dir,
                         **settings)
        self.server_name = options.srv_name
        self.logger = tornado.log.gen_log

        tornado.ioloop.IOLoop.configure(
            'tornado.platform.asyncio.AsyncIOMainLoop'
        )

        if options.use_mail_logging:
            self._set_mail_logging()

        if options.use_curl_http_client:
            self.log_debug(options.use_curl_http_client,
                           grep_label='use_curl_http_client')
            try:
                importlib.import_module('pycurl')
                AsyncHTTPClient.configure(
                    "tornado.curl_httpclient.CurlAsyncHTTPClient"
                )
            except ImportError:
                raise ImportError('Required package for CurlAsyncHTTPClient '
                                  'pycurl is missing')

        self.http_client = AsyncHTTPClient(
            max_clients=options.max_http_clients
        ) if create_http_client else None

        if options.use_mongo:
            try:
                self.mongo_pool = get_mongo_pool(
                    mongo_db_name=options.mongo_db_name,
                    mongo_user=options.mongo_user,
                    mongo_psw=options.mongo_psw,
                    mongo_auth_db_name=options.mongo_auth_db_name,
                    mongo_server=options.mongo_server,
                    mongo_port=options.mongo_port,
                    mongo_min_pool_size=options.mongo_min_pool_size,
                    mongo_max_pool_size=options.mongo_max_pool_size
                )
            except ModuleNotFoundError:
                self.mongo_pool = None
                raise
        else:
            self.mongo_pool = None
        self.event_writer = TorskelEventLogController()

    def init_srv(self):
        """
        Initializing an application on a port or socket depending on
         the settings
        :return: None
        """
        server_init(self)

    @staticmethod
    def get_secret_key() -> str:
        """
        Return secret key from options
        :return: str
        """
        return options.secret_key

    # ########################### #
    #  Validate params functions  #
    # ########################### #
    @staticmethod
    def _validate_options() -> bool:
        """
        Skeleton for validating options
        :return: bool
        """
        return True

    @staticmethod
    def _validate_path() -> bool:
        """
        Skeleton for validating paths
        :return: bool
        """
        return True

    @staticmethod
    def _validate_mail_params() -> bool:
        """
        Skeleton for validating email
        :return: bool
        """
        return True

    # ################# #
    #  Init with loop   #
    # ################# #

    def init_with_loop(self, loop=None):
        """
        Initialization with loop
        :param loop:
        :return:
        """

        if options.use_redis:
            self.log_info("Init Redis connection pool... ")
            self.log_info(f"ADDR={self.redis_addr} DB={self.redis_db}",
                          grep_label=INIT_REDIS_LABEL)
            self.log_info(f"MIN_POOL_SIZE={self.redis_min_con} "
                          f"MAX_POOL_SIZE={self.redis_max_con}",
                          grep_label=INIT_REDIS_LABEL)
            self.init_redis_pool(loop)
        if options.use_events_writer:
            self.log_info('Init events writer')
            event_writer = tornado.ioloop.PeriodicCallback(
                self.write_log_from_queue,
                options.writer_period
            )
            event_writer.start()

    # ############################# #
    #  Async Http-client functions  #
    # ############################# #
    @staticmethod
    def get_xmlrpc_server(xmlrpc_server,
                          max_connections=options.max_xmlrpc_clients):
        """
        return connection to xmlrpc server
        :param xmlrpc_server: server url
        :param max_connections: count of max connections
        :return: connection
        """
        res = None
        if options.use_xmlrpc:
            try:
                tornado_xmlrpc = importlib.import_module(
                    'tornado_xmlrpc.client')
                res = tornado_xmlrpc.ServerProxy(
                    xmlrpc_server,
                    AsyncHTTPClient(max_clients=max_connections)
                )
            except ImportError:
                raise ImportError(
                    'Required package tornado_xmlrpc is missing!'
                )

        return res

    async def http_request_post(self, url, body, **kwargs):
        """
        http request. Method POST
        :param url: url
        :param body: dict with POST-params
        param from_json: boolean, convert response to dict
        none_if_err return None if failed request, default True
        :return: response
        """
        from_json = kwargs.get('from_json', False)
        from_xml = kwargs.get('from_xml', False)
        log_timeout_exc = kwargs.get('log_timeout_exc', True)
        none_if_err = kwargs.get('none_if_err', True)
        self.log_debug(log_timeout_exc, grep_label='log_timeout_exc')
        res = None
        try:
            headers = None
            param_s = urlencode(body)

            res_fetch = await self.http_client.fetch(url, method='POST',
                                                     body=param_s,
                                                     headers=headers)

            res_s = res_fetch.body.decode(encoding="utf-8") \
                if res_fetch is not None else res_fetch

            res = res_s
            if from_json:
                res = json.loads(res_s)
            if from_xml:
                res = json.loads(json.dumps(xmltodict.parse(res_s)))
        except tornado.httpclient.HTTPError as exception:
            if exception.code == 599:
                if log_timeout_exc is True:
                    self.log_exc('http_request_get failed by timeout url = %s'
                                 % url)
                else:
                    self.log_debug('http_request_get failed by timeout')
            if none_if_err:
                res = None

        return res

    async def http_request_get(self, url, **kwargs):
        """
        http request. Method GET
        :param url: url
        param from_json: boolean, convert response to dict
        none_if_err return None if failed request, default True
        :return: response
        """
        from_json = kwargs.get('from_json', False)
        from_xml = kwargs.get('from_xml', False)
        log_timeout_exc = kwargs.get('log_timeout_exc', True)
        none_if_err = kwargs.get('none_if_err', True)
        self.log_debug(log_timeout_exc, grep_label='log_timeout_exc')
        res = None
        try:
            res_fetch = await self.http_client.fetch(url)
            res_s = res_fetch.body.decode(encoding="utf-8") \
                if res_fetch is not None else res_fetch

            res = res_s
            if from_json:
                res = json.loads(res_s)

            if from_xml:
                res = json.loads(json.dumps(xmltodict.parse(res_s)))
        except tornado.httpclient.HTTPError as exception:
            if exception.code == 599:
                if log_timeout_exc is True:
                    self.log_exc('http_request_get failed by timeout url = %s'
                                 % url)
                else:
                    self.log_debug('http_request_get failed by timeout')
            if none_if_err:
                res = None

        return res

    def _set_mail_logging(self, log_level=logging.ERROR):
        """
        Init SMTP log handler for sendig log to email
        :param log_level: log level
        :return:

        """
        if options.log_mail_user == '' and options.log_mail_psw == '':
            credentials_list = None
        else:
            credentials_list = [
                options.log_mail_user, options.log_mail_psw
            ]

        # TODO validate mail params try catch
        mail_logging = logging.handlers.SMTPHandler(
            mailhost=options.log_mail_host,
            fromaddr=options.log_mail_from,
            toaddrs=options.log_mail_to,
            subject=options.log_mail_subj,
            credentials=credentials_list
        )

        mail_logging.setLevel(log_level)
        self.logger.addHandler(mail_logging)

    async def write_log_from_queue(self) -> type(None):
        """
         Retrieves events from the queue.
         and performs the insert into the database
        """
        if self.mongo_pool is None and not options.use_mongo:
            self.log_err('Can not write event. '
                         'Connection to database is missing!')
        else:
            await self.event_writer.write_log_from_queue(
                self.mongo_pool,
                options.events_collection_name,
                bulk_mongo_insert
            )



from distutils.core import setup
from setuptools import find_packages

VERSION = '0.5.7'

setup(
    name='torskel',
    version=VERSION,
    packages=find_packages(),
    url='https://github.com/frostspb/torskel',
    license='MIT',
    author='Frostspb',
    description='Python package with skeleton of the base Tornado application',
    long_description="""Contains basic functions for logging,
    asynchronous redis and asynchronous http request out of box
    Only Python 3.7+""",
    keywords=["tornado"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3.7",
    ],
    install_requires=[
        'xmltodict>=0.11.0',
        'user-agents>=1.1.0',
        'tornado>=5.1',
        'pyjwt>=1.7.1',
        'python-jwt>=3.2.4',
    ],
)
